# Copyright 2013 tsuru-circus authors. All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from setuptools import setup, find_packages
from tsuru import __version__


setup(
    name="tsuru-circus",
    version=__version__,
    packages=find_packages(),
    description="Circus extensions for tsuru.",
    author="timeredbull",
    author_email="timeredbull@corp.globo.com",
    include_package_data=True,
    install_requires=["circus>=0.6.0", "requests", "honcho==0.2.0", "PyYAML==3.10"],
    tests_require=["nose", "mock"],
    test_suite="nose.collector"
)
